from pasteob import ErrorResponse
from pasteob.auth.logic import *

__all__ = ['BasicAuthLogic']

class BasicAuthLogic(AuthLogic):
    # if populate_req is True the environ keys 'REMOTE_USER' and 'AUTH_TYPE' will be
    # adjusted if the provided credentials are valid
    populate_req = True
    realm = None
    _username = None # these are what user provided -- they can be invalid!
    _password = None
    def __init__(self, req):
        assert self.realm, "BasicAuthLogic subclasses should define their realm"
        if req.authorization and req.authorization[0] == 'Basic':
            auth_data = req.authorization[1].strip().decode('base64')
            self._username, self._password = auth_data.split(':', 1)
            if self._username: # username can't be empty
                self.check_password()
            if self.logged_in and self.populate_req:
                req.remote_user = self._username
                #req.environ['AUTH_TYPE'] = 'basic'

    def check_password(self):
        raise NotImplementedError

    def _make_user(self):
        assert self.logged_in
        return User(self._username)

    def _login_now(self):
        assert not self.logged_in
        return ErrorResponse(401, www_authenticate=('Basic', {'realm': self.realm}))

    #def _login_url(self, return_to):
    #    return users.create_login_url(return_to)

    #def _logout_url(self, return_to):
    #    return users.create_logout_url(return_to)

