from webob.request import BaseRequest, environ_from_url#, environ_add_POST
from webob.response import Response as WebobResponse, iter_close
from mext.uri import AbsoluteUri
from mext.uri.encode import _new_quote_table, _qt_safe2, charmap_decode



__all__ = [
    'Request', 'Response', 'OverrideResponse',
    'wapp', 'wapp_mw',
]

#@@ override GET, POST, cookies, params

def _set_attrs(ob, kw):
    cls = ob.__class__
    for name, value in kw.iteritems():
        if not hasattr(cls, name):
            raise TypeError("Unexpected keyword: %s=%r" % (name, value))
        setattr(ob, name, value)

_qt_path = _new_quote_table(_qt_safe2, '', '?#')
def quote_path(s):
    return str(charmap_decode(s, 'strict', _qt_path)[0])


@property
def masked_prop(self):
    raise AttributeError("This property is masked by PasteOb")

class Request(BaseRequest):
    _charset_cache = None
    #charset = property(staticmethod(lambda: 'UTF-8'))
    charset = property(lambda self: 'UTF-8')


    @property
    def path(self):
        e = self.environ
        return quote_path(e['SCRIPT_NAME']+e['PATH_INFO'])

    @property
    def path_qs(self):
        e = self.environ
        r = quote_path(e['SCRIPT_NAME']+e['PATH_INFO'])
        qs = e.get('QUERY_STRING') # may be absent
        if qs:
            return r + '?' + qs
        return r


    @property
    def host_url(self):
        e = self.environ
        url = e['wsgi.url_scheme'] + '://'
        host = e.get('HTTP_HOST')
        if not host:
            url += e['SERVER_NAME']
            port = e['SERVER_PORT']
        elif ':' in host:
            host, port = host.split(':', 1)
            url += host
        else:
            return url+host

        if ((port == '80' and e['wsgi.url_scheme'] == 'http')
            or (port == '443' and e['wsgi.url_scheme'] == 'https')
        ):
            return url
        else:
            return url + (':%s' % port)

    @property
    def application_url(self):
        return self.host_url + quote_path(self.environ['SCRIPT_NAME'])

    @property
    def url(self):
        return self.host_url + self.path_qs

#     @property
#     def url_no_query(self):
#         return self.host_url + self.path
#     path_url = url_no_query # webob uses this name


    uri = property(lambda self: AbsoluteUri(self.url))
    app_uri = property(lambda self: AbsoluteUri(self.application_url))
    appdir_uri = property(lambda self: AbsoluteUri(self.application_url).as_dir())
    host_uri = property(lambda self: AbsoluteUri(self.host_url))

    @property
    def referrer_uri(self):
        r = self.referrer
        return AbsoluteUri(r) if r else None

    def __init__(self, env='/', **kw):
        if isinstance(env, str):
            env = environ_from_url(env)
        self.environ = env
        if kw:
            _set_attrs(self, kw)

    def get_response(self, app):
        status, headers, app_iter = self.call_application(app)
        return Response(status=status, headerlist=list(headers), app_iter=app_iter)
    __rlshift__ = get_response # resp = app<<req

    str_GET = str_POST = str_params = str_cookies = urlvars = masked_prop



class Response(WebobResponse):
    ctype = WebobResponse.content_type
    conditional_response = property()
    RequestClass = Request

    # when overriding __call__, override this as well
    def __lshift__(self, req):
        return self.__class__(
            status=self.status,
            headerlist=self.headerlist,
            app_iter=self.app_iter
        )

    _status = '200 OK'
    _environ = _request = None
    _body = _app_iter = _headers = None

    def __init__(self,
        body=None, status=None, headerlist=None, app_iter=None,
        ctype=None, charset=None, content_type=None,
        **kw
    ):
        if status is not None:
            self.status = status
        if ctype is None:
            ctype = content_type
        else:
            assert content_type is None

        if headerlist is not None:
            assert ctype is charset is None
            self._headerlist = headerlist
        elif ctype is None:
            assert charset is None
            self._headerlist = [('Content-Type', 'text/html; charset=UTF-8')]
            charset = 'UTF-8'
        else:
            if charset:
                ctype += '; charset=' + charset
            elif 'charset=' in ctype:
                pass
                # we can detect charset for ubody, but that's a very strange case
                #charset = ctype.rsplit('charset=', 1)[1]
            elif charset is not None:
                # pass charset='' or charset=False (anything false but not None)
                # to disable automatic utf-8 charset for text and xml responses
                pass
            elif (ctype.startswith('text/')
                or (ctype.startswith('application/') and ctype.endswith('xml'))
            ):
                # assert charset is None
                ctype += '; charset=UTF-8'
                charset = 'UTF-8'
            self._headerlist = [('Content-Type', ctype)]

        if app_iter is not None:
            assert body is None
            self._app_iter = app_iter
        else:
            body = body or ''
            if isinstance(body, unicode):
                assert charset, "Got unicode body but no charset"
                body = body.encode(charset)
            self._body = body
            assert not headerlist or ('Content-Length' not in self.headers)
            self._headerlist.append(('Content-Length', str(len(body))))
        if kw:
            _set_attrs(self, kw)

    def __call__(self, environ, start_response):
        start_response(self.status, self.headerlist)
        return self.app_iter

    def flat_app_iter(self):
        r = self._app_iter
        if r is None:
            return [self._body]
        if not isinstance(r, (list, tuple)):
            r = self._app_iter = list(r)
        return r


class OverrideResponse(BaseException):
    def __init__(self, app, comment=None):
        self.app = app
        self.comment = comment





class wapp(object):
    __slots__ = 'func'
    def __init__(self, func):
        self.func = func

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.func)

    def __get__(self, ob, type):
        try:
            get = self.func.__get__
        except:
            return self
        else:
            return self.__class__(get(ob, type))

    def __call__(self, env, start):
        try:
            app = self.func(Request(env))
            if app is None:
                app = self.error_404
        except OverrideResponse, exc:
            app = exc.app
        return app(env, start)


class wapp_mw(object):
    __slots__ = ('mwfunc', 'next_app', 'args', 'kw')
    def __new__(cls, *args, **kw):
        if len(args) == 1 and not kw:
            mwfunc = args[0]
            factory = lambda *args, **kw: cls(mwfunc, *args, **kw)
            factory.__name__ = 'wapp_mw(%r)' % mwfunc
            return factory
        return object.__new__(cls)

    def __init__(self, mwfunc, next_app, *args, **kw):
        self.mwfunc = mwfunc
        self.next_app = next_app
        self.args = args
        self.kw = kw

    def __repr__(self):
        kwrepr = ''.join(
            map(', %r'.__mod__, self.args)
            + map(', %s=%r'.__mod__, self.kw.items())
        )
        return '%s(%r, %r%s)' % (self.__class__.__name__, self.mwfunc, self.next_app, kwrepr)

    @wapp
    def __call__(self, req):
        return self.mwfunc(req, self.next_app, *self.args, **self.kw)

