import sys
from pasteob import *
from pasteob.htmlgen import *

from pygments.formatters import HtmlFormatter
from pygments import highlight as pygments_highlight
from pygments.lexers import PythonLexer

try:
    from mext.html.syntax_hilight import MyStyle as my_style
except:
    from pygments.styles.default import DefaultStyle as my_style

__all__ = ['show_module_source_app']


@wapp
@staticmethod
def show_module_source_app(req):
    module_name = req.path_info.lstrip('/')
    module = sys.modules.get(module_name)
    if module is not None:
        if not hasattr(module, '__file__'):
            return Response('<builtin module %s>' % module_name, ctype='text/plain')
        filename, source = get_source(module)
        html = HTML
        html += TITLE[module_name, ' - ', filename]
        html += highlight_src(source, linenos=True)
        return HtmlResponse(html)


def highlight_src(
    code,
    linenos=False,
    lineanchors='code',
    cssclass='highlight'
):
    lexer = PythonLexer()
    formatter = HtmlFormatter(
        linenos=linenos,
        lineanchors=lineanchors,
        cssclass=cssclass,
        style=my_style,
    )
    css = STYLE[formatter.get_style_defs('.'+cssclass)]
    ht_code = raw_html(pygments_highlight(code, lexer, formatter))
    return [css, ht_code]

def get_source(module):
    filename = module.__file__
    if filename[-4:] in ('.pyc', '.pyo'):
        filename = filename[:-1]
    elif filename.endswith('$py.class'):
        filename = '%s.py' % filename[:-9]
    source = open(filename, 'rb').read()
    return filename, source



