from webob.util import status_reasons
from pasteob.base import wapp, Response
from mext.uri.parse import is_absolute

__all__ = ['ErrorResponse', 'RedirectResponse']

_ctype_text = 'text/plain'
_ctype_html = 'text/html'

_string_codes = {
    'auth': 401,
    'forbidden': 403,
    'method': 405,
}

class ErrorResponse(object):
    def __init__(self, status, message=None, comment=None, **kw):
        if not isinstance(status, int):
            status = _string_codes[status]
        self.status = status
        self.title = u"%s %s" % (status, status_reasons[status])
        self.message = unicode(message or u'')
        self.comment = unicode(comment or u'')
        self.kw = kw

    @wapp
    def __call__(self, req):
        ctype = req.accept.best_match([_ctype_text, _ctype_html], _ctype_text)
        if ctype == _ctype_text:
            return self.text_response
        else:
            return self.html_response

    @property
    def text_response(self):
        body = self.title
        if self.message:
            body += u'\n' + self.message
        if self.comment:
            body += u'\n\n' + self.comment
        return Response(body, self.status, ctype=_ctype_text, **self.kw)

    @property
    def html_response(self):
        title = escape_text(self.title)
        body = u"<title>%s</title><h2>%s</h2>%s" % (
            title, title,
            escape_text(self.message)
        )
        if self.comment:
            body += u'\n<!--\n%s\n-->' % escape_text(self.comment)
        return Response(body, self.status, ctype=_ctype_html, **self.kw)

_redirect_types = {
    'permanent': 301,
    'perm': 301,
    #'found': 302,
    'see other': 303,
    'see': 303,
    'temp': 307,
}

class RedirectResponse(object):
    def __init__(self, status, location, message=None):
        if not isinstance(status, int):
            status = _redirect_types[status]
        self.status = status
        self.location = unicode(location)
        self.is_absolute = is_absolute(self.location)
        self.message = unicode(message or u"%s %s" % (status, status_reasons[status]))
        if self.is_absolute:
            self.response = self.make_response(self.location)

    def make_response(self, location):
        return Response(
            u'%s\n%s' % (self.message, location),
            self.status,
            ctype=_ctype_text,
            charset='UTF-8',
            location=location,
        )


    @wapp
    def __call__(self, req):
        if self.is_absolute:
            return self.response
        location = unicode(req.app_uri.join(self.location))
        return self.make_response(location)



def escape_text(text):
    return text.replace('&', '&amp;').replace('>', '&gt;').replace('<', '&lt;')
